@@ -63,6 +63,7 @@ impl<'a, Aux> Vm<'a, Aux> {
         self.runtime_data.global_vars.get(name.0 as usize).cloned()
     }
 
+    #[must_use]
     pub fn with_max_iter(mut self, max_iter: u64) -> Self {
         self.max_instr = max_iter;
         self
@@ -146,6 +147,24 @@ impl<'a, Aux> Vm<'a, Aux> {
         self.runtime_data.init_table()
     }
 
+    /// Initializes a new string owned by this VM instance
+    pub fn init_string(&mut self, payload: &str) -> Result<StrPointer, ExecutionError> {
+        unsafe {
+            let layout = std::alloc::Layout::from_size_align(4 + payload.len(), 4).unwrap();
+            let mut ptr = self
+                .runtime_data
+                .memory
+                .alloc(layout)
+                .map_err(|_| ExecutionError::OutOfMemory)?;
+
+            let result: *mut u8 = ptr.as_mut();
+            std::ptr::write(result as *mut u32, payload.len() as u32);
+            std::ptr::copy(payload.as_ptr(), result.add(4), payload.len());
+
+            Ok(StrPointer(ptr.as_ptr()))
+        }
+    }
+
     /// This mostly assumes that program is valid, produced by the compiler.
     /// As such running non-compiler emitted programs is very un-safe
     pub fn run(&mut self, program: &CaoProgram) -> Result<(), ExecutionError> {
@@ -198,13 +217,14 @@ impl<'a, Aux> Vm<'a, Aux> {
                     instr_execution::begin_repeat(self)?;
                 }
                 Instruction::Repeat => {
-                    if instr_execution::repeat(self)? >= 0 {
+                    if instr_execution::repeat(self)? {
                         instr_execution::instr_jump(
                             &mut instr_ptr,
                             program,
                             &mut self.runtime_data,
                         )?;
                     } else {
+                        self.stack_push(false)?; // assumes that the next instruction is GotoIfTrue
                         instr_ptr += instruction_span(Instruction::CallLane) as usize - 1;
                     }
                 }
@@ -219,9 +239,9 @@ impl<'a, Aux> Vm<'a, Aux> {
                             &mut self.runtime_data,
                         )?;
                     } else {
-                        // add the span of the jump instruction metadata to the instr_ptr
-                        // to skip this instruction
                         self.stack_push(false)?; // assumes that the next instruction is GotoIfTrue
+                                                 // add the span of the jump instruction metadata to the instr_ptr
+                                                 // to skip this instruction
                         instr_ptr += instruction_span(Instruction::CallLane) as usize - 1;
                     }
                 }
